package de.w665.sharepulse.rest.mappings;

import com.google.gson.JsonObject;
import de.w665.sharepulse.model.FileUpload;
import de.w665.sharepulse.rest.ApiRestController;
import de.w665.sharepulse.service.FileService;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@Slf4j
@RestController
public class Upload extends ApiRestController {

    private final FileService fileService;

    @Autowired
    public Upload(FileService fileService) {
        this.fileService = fileService;
    }

    // Currently testing
    @PostMapping("/upload")
    public ResponseEntity<String> getUpload(@RequestParam("file") MultipartFile file, HttpServletRequest request,
                                            @RequestParam(value = "passwordProtected", required = false) Boolean passwordProtected,
                                            @RequestParam(value = "singleDownload", defaultValue = "false") boolean singleDownload,
                                            @RequestParam(value = "fileDescription", required = false) String fileDescription) {

        // TODO: Handle shortStorage0

        if (file.isEmpty()) {
            log.debug("User tried to upload an empty file. IP: " + request.getRemoteAddr());
            return new ResponseEntity<>("Please select a file to upload.", HttpStatus.NOT_ACCEPTABLE);
        }

        FileUpload fileUpload = fileService.processUploadedFile(file, request.getRemoteAddr(), passwordProtected, singleDownload, fileDescription);
        log.debug("User uploaded file " + file.getOriginalFilename() + " from IP " + request.getRemoteAddr() + " successfully.");

        JsonObject response = new JsonObject();
        response.addProperty("fileId", fileUpload.getFileId());
        response.addProperty("fileName", fileUpload.getFileName());
        response.addProperty("message", "File " + file.getOriginalFilename() + " uploaded successfully!");
        response.addProperty("passwordProtected", fileUpload.isPasswordProtected());
        response.addProperty("singleDownload", fileUpload.isSingleDownload());
        response.addProperty("uploadDate", fileUpload.getUploadDate().toString());
        response.addProperty("password", fileUpload.getDownloadPassword());

        return new ResponseEntity<>(response.toString(), HttpStatus.OK);
    }
}
